@@ -3,7 +3,7 @@
  *  Interface to GiNaC's non-commutative products of expressions. */
 
 /*
- *  GiNaC Copyright (C) 1999-2015 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2019 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -49,30 +49,30 @@ public:
 	      const ex & f4, const ex & f5);
 	ncmul(const ex & f1, const ex & f2, const ex & f3,
 	      const ex & f4, const ex & f5, const ex & f6);
-	ncmul(const exvector & v, bool discardable=false);
-	ncmul(std::auto_ptr<exvector> vp);
+	ncmul(const exvector & v);
+	ncmul(exvector && v);
 
 	// functions overriding virtual functions from base classes
 public:
-	unsigned precedence() const {return 50;}
-	bool info(unsigned inf) const;
-	int degree(const ex & s) const;
-	int ldegree(const ex & s) const;
-	ex expand(unsigned options=0) const;
-	ex coeff(const ex & s, int n=1) const;
-	ex eval(int level=0) const;
-	ex evalm() const;
-	exvector get_free_indices() const;
-	ex thiscontainer(const exvector & v) const;
-	ex thiscontainer(std::auto_ptr<exvector> vp) const;
-	ex conjugate() const;
-	ex real_part() const;
-	ex imag_part() const;
+	unsigned precedence() const override {return 50;}
+	bool info(unsigned inf) const override;
+	int degree(const ex & s) const override;
+	int ldegree(const ex & s) const override;
+	ex expand(unsigned options=0) const override;
+	ex coeff(const ex & s, int n=1) const override;
+	ex eval() const override;
+	ex evalm() const override;
+	exvector get_free_indices() const override;
+	ex thiscontainer(const exvector & v) const override;
+	ex thiscontainer(exvector && v) const override;
+	ex conjugate() const override;
+	ex real_part() const override;
+	ex imag_part() const override;
 
 protected:
-	ex derivative(const symbol & s) const;
-	unsigned return_type() const;
-	return_type_t return_type_tinfo() const;
+	ex derivative(const symbol & s) const override;
+	unsigned return_type() const override;
+	return_type_t return_type_tinfo() const override;
 	
 	// new virtual functions which can be overridden by derived classes
 	// none
@@ -83,7 +83,7 @@ protected:
 	void do_print_csrc(const print_context & c, unsigned level) const;
 	size_t count_factors(const ex & e) const;
 	void append_factors(exvector & v, const ex & e) const;
-	std::auto_ptr<exvector> expandchildren(unsigned options) const;
+	exvector expandchildren(unsigned options) const;
 public:
 	const exvector & get_factors() const;
 };
